// =========================================================================
// Program Name : FFF.c
// Author       : Bill Buckels
// Date         : September 25, 1995
// Purpose      : Conversion Utility to convert from a flat binary font file
//                to an FFF (Free Font format) File for use with the
//                VESALIB graphics font routines
//
// =========================================================================

#include <stdio.h>
#include <fcntl.h>
#include <dos.h>
#include <bios.h>
#include <io.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

int wPadFactor=0;

typedef struct
    {
    char szFileType[8]      ;        // FREEFONT
    unsigned int wWidth     ;        // width
    unsigned int wHeight    ;        // height
    unsigned long dwHeadSize;        // header size - offset to fontdata
    unsigned int wBufSize   ;        // buffer size to allocate for font
    }FontHeaderBlock;

FontHeaderBlock fhb;
char *lpFontData;
char szFontDesc[128]="\0";

int loaddumpfont(char *lpszBuf)
{
    FILE *fp,*fp2;
    int i, j, k;
    unsigned wPacket;

    if((fp = fopen(lpszBuf,"rb"))==NULL)return -1;
    printf("Reading... %s\n",lpszBuf);
    j= (fhb.wWidth/8);
    if(fhb.wWidth%8!=0)j++;
    wPacket = j * fhb.wHeight;
    fhb.wBufSize   = 256 +( 256 * wPacket);
    fhb.dwHeadSize = 1L + sizeof(FontHeaderBlock) + strlen(szFontDesc);

    for(i=0;lpszBuf[i]!=0;)
    {
        if(lpszBuf[i]=='.')
        {
            lpszBuf[i]=0;
            break;
        }
        i++;
    }
    strcat(lpszBuf,".FFF");

    if((fp2 = fopen(lpszBuf,"wb"))==NULL)
    {
        perror(lpszBuf);
        return -2;
    }

    printf("Writing... %s\n",lpszBuf);
    fwrite(&fhb,sizeof(FontHeaderBlock),1,fp2);
    fwrite(szFontDesc,strlen(szFontDesc)+1,1,fp2);

    lpFontData= malloc(wPacket+1);
    memset(lpFontData,0,wPacket);
    lpFontData[wPacket]=fhb.wWidth;
    for(i=0;i<256;i++)
    {

      fread(lpFontData,wPacket-(fhb.wHeight*wPadFactor),1,fp);
      if(wPadFactor)
      {
       for(k=0;k<fhb.wHeight;k++)
       {
          fwrite((char *)&lpFontData[k*(j-1)],j-1,1,fp2);
          fputc(0,fp2);
       }
       fputc(lpFontData[wPacket],fp2);
      }

      else
      {
        fwrite(lpFontData,wPacket+1,1,fp2);
       }
    }
    fclose(fp);
    fclose(fp2);
    puts("Done!");
    return 0;
}


char *lpszUsage[]={
"Binary Flat Font to Free Font File Format Conversion Utility.",
"Input  : 256 Character Binary Font without header.",
"Output : 256 Character Free Font Format (.FFF) File.",
"Usage  : \"FFF <infile.bin> \"Description in Quotes\" width height\"",
"Option : \"FFF <bin> <desc> <desired width> <height> PAD\"",
"         The purpose of the pad option is to expand the binary width",
"         of the output file cells by one byte. This is helpful when",
"         no space has been allowed between characters in the input",
"         file causing the characters to draw too close together.",
""};

int main(int argc, char **argv)
{
    char szBuf[81];
    int i;

    puts("FFF(C) Bill Buckels Copyright 1995");
    if(argc==5||argc==6)
    {
        strcpy((char *)&fhb.szFileType[0],"FREEFONT");
        strcpy(szFontDesc,argv[2]);
        fhb.wWidth =atoi(argv[3]);
        fhb.wHeight=atoi(argv[4]);
        if(strcmpi(argv[5],"PAD")==0)wPadFactor++;
        strcpy(szBuf,argv[1]);
        if(loaddumpfont(szBuf)==-1)perror(argv[1]);
    }
    else
    {
      for(i=0;lpszUsage[i][0]!=0;i++)puts((char *)&lpszUsage[i][0]);

    }
    return 0;
}
