// PC Speaker Sound Routine
// Mouse Detection Routine and Mouse Button Press Routine

#define MOUSELIB
#include "vesalib.h"

// freq of sound in Hertz
// duration of sound 18.2 ticks per second

int sound(int freq,int tlen)
{
    union REGS inregs,outregs;
    unsigned frdiv = 1331000L;
    int seed,hiseed,hold=0,setting;

    if(freq<1)return freq;       // avoid divide by zero
    frdiv /= freq;               // timer divisor

    if(freq!=SILENCE)
    {
    outp(0x43,0xB6) ;            // write timer mode register
    outp(0x42,frdiv & 0x00FF);   // write divisor LSB
    outp(0x42,frdiv >> 8);       // write divisor MSB

    setting= inp(0x61);          // get current port B setting
    outp(0x61,setting | 0x03);   // turn speaker on
    }

    if(tlen>0)
    {
       tlen=((tlen*1000)/182);   // convert from 18.2 ticks to 100ths
       inregs.x.ax= TIMEOUT;
       int86(0x21,&inregs,&outregs);
       seed=(outregs.x.dx)&0xff;

    while(hold<tlen)
     {
       inregs.x.ax = TIMEOUT;
       int86(0x21,&inregs,&outregs);
       if(seed!=(outregs.x.dx&0xff))
        {
          hiseed= (outregs.x.dx)&0xff;
          if(hiseed<seed)hiseed+=100;
          hold+=(hiseed-seed);
          seed =(outregs.x.dx)&0xff;
          }
          }
          }

   if(freq!=SILENCE)
   {
    // restore port B setting
    outp(0x61,setting);
    }
    return 0;
}

void keyclick(void)
{
     sound(523,1);
     sound(659,1);
     sound(784,1);
}

void bronx(void)
{
     sound(60,4);
     sound(40,8);
}

void bell()
{
     sound(220,1);
     sound(440,1);
}

int mouse = FALSE;
// check for mouse button down
// wait until mouse button up
int mousebutton()
{
    int m2;

    if(mouse==FALSE)return FALSE;        // no mouse? outa' here.
    _asm{
         mov ax,3
         int 33h
         mov m2,bx
        }
    if(m2==0)return FALSE;               // buttons up? outa' here.

     while(m2!=0)                        // button down? wait for button up.
     {
       _asm{
        mov ax,3
        int 33h
        mov m2,bx
        }
      }

return TRUE;                             // signal button down to application.
}

int initmouse()
{
    union REGS inregs, outregs;
    struct SREGS segregs;
    long address;
    unsigned char first_byte;

    // check for microsoft compatible mouse driver
    inregs.x.ax = 0x3533;
    intdosx ( &inregs, &outregs, &segregs );
    address = (((long) segregs.es) << 16) + (long) outregs.x.bx;
    first_byte = * (long far *) address;
    if ((address == 0) || (first_byte == 0xcf))return (mouse = FALSE);

    // check to see if the mouse is responding
    inregs.x.ax=0;
    int86(0x33,&inregs,&outregs);
    if(outregs.x.ax==0)return(mouse = FALSE);
    else return (mouse = TRUE);
}

