// =========================================================================
// Program Name : VesaClip.c
// Author       : Bill Buckels
// Date         : September 25, 1995
// Purpose      : Create 256 color pcx clips with load coordinates saved.
//                Hardware Independent VESA Super VGA 256 Color
//                 Utility Program using the VESALIB Graphics Library.
//                This is useful for creating image fragments for
//                 loading at screen coordinates over a background
//                 using the xmin, xmax, ymin, ymax fields of the pcx.
//
// =========================================================================

#include "vesalib.h"

int VesaClip(char *lpszInFile, char *lpszOutFile)
{
    int status,c,topx,topy,botx,boty;
    int iXLimit, iYLimit;
    uint wUnits,x,y;
    uchar *ptr,temp;

    // start half-way if positional references are not saved
    iXLimit =  gib.wDefaultWidth-1;
    iYLimit =  gib.wDefaultHeight-1;
    sw.iTopx =  iXLimit/4;
    sw.iBotx =  iXLimit - sw.iTopx;
    sw.iTopy =  iYLimit/4;
    sw.iBoty =  iYLimit - sw.iTopy;

    // if the file is not a pcx, try to load it as a Windows BitMap
    if((status = LoadPcx(lpszInFile))!=SUCCESS)
    {
        if(status!=99)return status;
        status = LoadVesaBitMap(lpszInFile);
        if(status!=SUCCESS)return status;
        memset(&pcxheader,0,sizeof(PCXHEADER));
        pcxheader.zsoft     = 10  ;
        pcxheader.version   = 5   ;
        pcxheader.pixbits   = 8   ;
        pcxheader.no_planes = 1   ;
        ptr  = (char *)&sw.Pal[0];
        *ptr = 12;
        for(y=0;y<256;y++)
        {
          for(x=0;x<3;x++)
          {
          *ptr++;
          temp=gib.aRgbInfo[y][x];
          *ptr=(temp<<2);

           }
        }
    }

    // if we are offset in the frame
    // assume we have saved positional references
    if(pcxheader.xmin!=0 || pcxheader.ymin!=0)
    {
       sw.iTopx=pcxheader.xmin;
       sw.iTopy=pcxheader.ymin;
       sw.iBotx=pcxheader.xmax;
       sw.iBoty=pcxheader.ymax;
    }

    topx = sw.iTopx;
    topy = sw.iTopy;
    botx = sw.iBotx;
    boty = sw.iBoty;
    XBox(topx,topy,botx,boty);
    wUnits = 1;

    for(;;)
    {

      if((c=getch())==ESCKEY)break;
      if(c==ENTERKEY)
      {
        XBox(topx,topy,botx,boty);
        c=SavePcx(lpszOutFile);
        break;
      }
      if(c>48 && c <58)
      {
        wUnits = c-48;
        continue;
      }

      if(c==FUNCKEY)
      {

        c=getch();
        status=shiftstatus();
        switch(c)
        {
            case LEFTARROW:
                            if(status==LEFTSHIFT)sw.iTopx -=wUnits;
                            if(status==RIGHTSHIFT)sw.iBotx-=wUnits;
                            if(status==FALSE)
                            {
                                sw.iTopx-=wUnits;
                                sw.iBotx-=wUnits;
                                while(sw.iTopx<0)
                                {
                                   sw.iTopx++;
                                   sw.iBotx++;
                                }
                            }
                            if(sw.iTopx<0)sw.iTopx=0;
                            if(sw.iBotx<=sw.iTopx)sw.iBotx=sw.iTopx+1;
                            break;

            case RIGHTARROW:
                            if(status==LEFTSHIFT)sw.iTopx+=wUnits;
                            if(status==RIGHTSHIFT)sw.iBotx+=wUnits;
                            if(status==FALSE)
                            {
                                sw.iBotx+=wUnits;
                                sw.iTopx+=wUnits;
                                while(sw.iBotx>iXLimit)
                                {
                                    sw.iBotx-=1;
                                    sw.iTopx-=1;
                                }
                            }
                            if(sw.iBotx>iXLimit)sw.iBotx=iXLimit;
                            if(sw.iTopx>=sw.iBotx)sw.iTopx=sw.iBotx-1;
                            break;
            case UPARROW:
                            if(status==LEFTSHIFT)sw.iTopy -=wUnits;
                            if(status==RIGHTSHIFT)sw.iBoty-=wUnits;
                            if(status==FALSE)
                            {

                                sw.iTopy-=wUnits;
                                sw.iBoty-=wUnits;
                                while(sw.iTopy<0)
                                {
                                  sw.iTopy++;
                                  sw.iBoty++;
                                }
                            }
                            if(sw.iTopy<0)sw.iTopy=0;
                            if(sw.iBoty<=sw.iTopy)sw.iBoty=sw.iTopy+1;
                            break;
            case DOWNARROW:
                            if(status==LEFTSHIFT)sw.iTopy+=wUnits;
                            if(status==RIGHTSHIFT)sw.iBoty+=wUnits;
                            if(status==FALSE)
                            {
                                sw.iBoty+=wUnits;
                                sw.iTopy+=wUnits;
                                while(sw.iBoty>iYLimit)
                                {
                                    sw.iBoty--;
                                    sw.iTopy--;
                                }
                            }
                            if(sw.iBoty>iYLimit)sw.iBoty=iYLimit;
                            if(sw.iTopy>=sw.iBoty)sw.iTopy=sw.iBoty-1;
                            break;
            }

      }

      if(sw.iTopx!=topx||sw.iTopy!=topy||sw.iBotx!=botx||sw.iBoty!=boty)
      {
            XBox(topx,topy,botx,boty);
            topx = sw.iTopx;
            topy = sw.iTopy;
            botx = sw.iBotx;
            boty = sw.iBoty;
            XBox(topx,topy,botx,boty);
      }
      else
      {
        printf("%c",7);
      }
    }

    SetVesaMode(gib.wSaveMode);
    puts("VESACLIP(C) Copyright by Bill Buckels 1995");
    if(c==ESCKEY)printf("%s not saved... exiting!\n",lpszOutFile);
    else printf("%s saved...\n",lpszOutFile);

return SUCCESS;
}

int VesaSession(char *lpInStr,char *lpOutStr,int iWidth, int iHeight)
{
    int iRetVal;

    gib.wDefaultWidth  = SVGAWIDTH;
    gib.wDefaultHeight = SVGAHEIGHT;
    gib.wDefaultPlanes = SVGAPLANES;
    gib.wDefaultBits   = SVGABITS;

    if(iWidth>0 )gib.wDefaultWidth  = iWidth;
    if(iHeight>0)gib.wDefaultHeight = iHeight;

    if((gib.wVesaMode =
        FindVesaMode(gib.wDefaultWidth,gib.wDefaultHeight,
                     gib.wDefaultPlanes,gib.wDefaultBits))==FALSE)
                                                      return UNAVAILABLE;

    gib.wSaveMode = GetVesaMode();
    if(SetVesaMode(gib.wVesaMode)!=gib.wVesaMode)
      {
         SetVesaMode(gib.wSaveMode); return UNSUPPORTED;
      }

    if((iRetVal=InitVesaSession())==SUCCESS)
    {
       iRetVal = VesaClip(lpInStr, lpOutStr);
    }
    if(iRetVal!=SUCCESS)SetVesaMode(gib.wSaveMode);

return iRetVal;
}

char *lpTitle=
"VesaClip(C) Bill Buckels Copyright 1995\n"
"Hardware Independent VESA Super VGA 256 Color Utility Program\n"
"To Create 256 color pcx clips with load coordinates saved.";

int main(int argc, char **argv)
{
    int i,iRetVal,iWidth,iHeight;

    // If a VESA driver is not loaded,
    // There is no point in continuing.
    if((iRetVal = GetVgaInfo())!=SUCCESS)
    {
      puts(lpTitle);
      switch(iRetVal)
      {
      case INITFAIL:
            printf("VESA Super VGA Information not supported.\n");
            return 1;
      case EXECFAIL:
            printf("VESA Super VGA Information failed.\n");
            return 2;
      }

    }

    // if no file name was entered, exit
    iWidth  = 0;
    iHeight = 0;
    if(argc<3)
    {
   puts(lpTitle);
   printf("Usage \"VesaClip in.fil out.pcx (widthoption) (heightoption)\"\n");
   return 4;
    }
    if(argc>4)
    {
      iWidth=atoi(argv[3]);
      iHeight=atoi(argv[4]);
    }

    // Execute the vesa session...
    // if there is a problem, report the problem as an exit code
    if((iRetVal=VesaSession(argv[1],argv[2],iWidth,iHeight))!=SUCCESS)
    {
      puts(lpTitle);
      printf("Error Code %d\n",iRetVal);
    }

return iRetVal;
}
