// =========================================================================
// Program Name : VesaFont.c
// Author       : Bill Buckels
// Date         : September 25, 1995
// Purpose      : Demonstration/Utility Program
//                Hardware Independent VESA Super VGA 256 Color FNT
//                Font File Loader using the VESALIB Graphics Library
//                Routines for VESA 256 Color Mode
//
// =========================================================================

#include "vesalib.h"

#define VESABLUE   1
#define VESAGREEN  10
#define VESACYAN   11
#define VESARED    12
#define VESAYELLOW 14
#define VESAWHITE  15


int VesaFontTest(char *lpStr)
{
    FILE *fp;
    int i,x,y;
    char buf[128];

    x=(gib.wDefaultWidth/2)-1;
    y=gib.wDefaultHeight/8;

    LoadPalette();
    ClearScreen(VESABLUE);
    if((i=LoadVesaFont(lpStr))!=0)return i;

    if((fp = fopen(lpStr,"rb"))==NULL)return -1;
    fseek(fp,(long)sizeof(FontHeaderBlock),SEEK_SET);
    fread(buf,127,1,fp);
    fclose(fp);
    buf[127]=0;
    DrawVesaFont(buf,x,y*4,'M',VESAWHITE);

    DrawVesaFont("ABCDEFGHIJKLMNOPQRSTUVWXYZ",x,y,'M',VESARED);
    DrawVesaFont("abcdefghijklmnopqrstuvwxyz",x,y*2,'M',VESAYELLOW);

    sprintf(buf,"This is %s Font.",lpStr);
    DrawVesaFont(buf,x,y*3,'M',VESAWHITE);
    DrawVesaFont("the quick brown fox jumped over the lazy dogs",
                  x,y*5,'M',VESACYAN);
    DrawVesaFont("THE QUICK BROWN FOX JUMPED OVER THE LAZY DOGS",
                  x,y*6,'M',VESACYAN);
    DrawVesaFont("1234567890",x,y*7,'M',VESAGREEN);
    DrawBox(10,10,
            gib.wDefaultWidth-11,
            gib.wDefaultHeight-11,
            VESAWHITE);

    if(getch()==0)getch();

return SUCCESS;
}


int VesaSession(char *lpStr,int iWidth, int iHeight)
{
    int iRetVal;

    gib.wDefaultWidth  = SVGAWIDTH;
    gib.wDefaultHeight = SVGAHEIGHT;
    gib.wDefaultPlanes = SVGAPLANES;
    gib.wDefaultBits   = SVGABITS;

    if(iWidth>0 )gib.wDefaultWidth  = iWidth;
    if(iHeight>0)gib.wDefaultHeight = iHeight;

    if((gib.wVesaMode =
        FindVesaMode(gib.wDefaultWidth,gib.wDefaultHeight,
                     gib.wDefaultPlanes,gib.wDefaultBits))==FALSE)
                                                      return UNAVAILABLE;

    gib.wSaveMode = GetVesaMode();
    if(SetVesaMode(gib.wVesaMode)!=gib.wVesaMode)
      {
         SetVesaMode(gib.wSaveMode); return UNSUPPORTED;
      }

    if((iRetVal=InitVesaSession())==SUCCESS)
    {
       iRetVal = VesaFontTest(lpStr);
    }
    SetVesaMode(gib.wSaveMode);


return iRetVal;
}

char *lpTitle=
"VESAFONT(C) Bill Buckels Copyright 1995\n"
"Hardware Independent FNT raster font file loader";

int main(int argc, char **argv)
{
    FILE *fp;
    char szBuf[81];
    int i,iRetVal,iWidth,iHeight;

    // If a VESA driver is not loaded,
    // There is no point in continuing.
    if((iRetVal = GetVgaInfo())!=SUCCESS)
    {
      puts(lpTitle);

      switch(iRetVal)
      {
      case INITFAIL:
            printf("VESA Super VGA Information not supported.\n");
            return 1;
      case EXECFAIL:
            printf("VESA Super VGA Information failed.\n");
            return 2;
      }

    }

    // if no file name was entered
    // prompt for a file name.
    iWidth  = 0;
    iHeight = 0;
    if(argc<2)
    {
    puts(lpTitle);
    printf("Usage \"VESAFONT any.fnt (widthoption) (heightoption)\"\n");
    printf("\nEnter FNT File Name : ");gets(szBuf);
    if(szBuf[0]==0)return 4;
    }
    else
    {
    strcpy(szBuf,argv[1]);
    if(argc>3)
      {
      iWidth=atoi(argv[2]);
      iHeight=atoi(argv[3]);
      }
    }

    // Execute the vesa session...
    // if there is a problem, report the problem as an exit code
    if((iRetVal=VesaSession(szBuf,iWidth,iHeight))!=SUCCESS)
    {
      puts(lpTitle);
      printf("Error Code %d\n",iRetVal);
    }

return iRetVal;
}
