// =========================================================================
// Program Name : VesaLib2.c
// Author       : Bill Buckels
// Date         : September 25, 1995
// Purpose      : Core Graphics Library Routines for VESA 256 Color Mode
//
// =========================================================================
// Description  : Additional Routines To Be Used With The Routines In
//                VesaLib.c
//
//             1. VesaLib2.c is written in Large Model MSC 6.00a.
//                Stack Checking is turned off.
//
//             2. See VesaLib.c for licencing and related information.
//                The Same Conditions of Use That Apply to VesaLib.c
//                Also Apply to VesaLib2.C.
//
//             2.1 With the exception of the restriction against publicly
//                 declaring that you own my work, and the restriction that
//                 my work can't be resold as part of a programmer's toolkit
//                 without my permission, you may do with this code whatever
//                 pleases you.
//
//             2.2 If you can't abide by these conditions of use, you may
//                 not use or distribute this code or the programs
//                 produced from it.
//
//             2.3 If you chose to disregard these conditions of use,
//                 may you be cursed by the loss of your creative ability
//                 as punishment for your blatant plagiarism, and I will
//                 also prosecute you for profit if I catch you and if it
//                 pleases me to do so.
//
//                 Bill Buckels
//                 589 Oxford Street
//                 Winnipeg, Manitoba, Canada 23M 3J2
//                 Voice - (204) 489-1405
//
// =========================================================================

#define VESALIB2
#include "vesalib.h"

// =========================================================================
// Miscellaneous Functions
// =========================================================================

int shiftstatus()
{
    flag.status= keystatus[0];
    if(flag.bit.right_shift)return RIGHTSHIFT;
    if(flag.bit.left_shift)return  LEFTSHIFT;

return FALSE;
}


void HXLine(uint x1, uint x2, uint y)
{
    uint x,i;
    uchar far *ptr;

    if(x2<x1)
    {
        x=x1;x1=x2;x2=x;
    }
    x2++;
    i=0;
    for(x=x1;x<x2;x++)i++;
    ptr = (char *)&gib.lpScanLine[0];
    VesaRaster(ptr,x1,y,i,READFLAG);
    for(x=0;x<i;x++)ptr[x]^=0xff;
    VesaRaster(ptr,x1,y,i,WRITEFLAG);
}

void VXLine(unsigned y1, unsigned y2, unsigned x)
{
    unsigned y;
    y2++;
    for(y=y1;y<y2;y++)XDot(x,y);
}


void XBox(unsigned x1,unsigned y1,unsigned x2,unsigned y2)
{
    HXLine(x1,x2,y1);
    y1++;
    y2--;
    VXLine(y1,y2,x1);
    VXLine(y1,y2,x2);
    y2++;
    HXLine(x1,x2,y2);
}

void XBlox(unsigned x1,unsigned y1,unsigned x2,unsigned y2)
{
    unsigned y;
    y2++;
    for(y=y1;y<x2;y++)
    {
      HXLine(x1,x2,y);
    }
}

// =========================================================================
// Pcx Functions
// =========================================================================

// PCX write functions start here...
// ancillary function to write an encoded byte and return bytes writtten
int PcxPut(FILE *PcxFile,unsigned char byt,unsigned char cnt)
{
    if(cnt)
    {
        if((cnt==1)&& (0xc0 != (0xc0 &byt)))
            {
                if(EOF == fputc((int)byt,PcxFile))return 0;
                return 1;
            }
         if(EOF==fputc((int)0xc0|cnt,PcxFile))return 0;
         if(EOF==fputc((int)byt,PcxFile))return 0;
         return 2;
   }

return 0;
}

// write a pcx line
// this works for all types of pcx files...
int WritePcxLine(FILE *PcxFile, unsigned char *ptr, int bytes)
{
    unsigned char this,last;
    int srcindex,i;
    register int total;
    register unsigned char count;

    total=0;
    last = *(ptr); count=1;
    for(srcindex=1;srcindex<bytes;srcindex++)
        {
      this= *(++ptr);
      if(this==last)
        {
        count++;
        if(count==63)
            {
            if(!(i=PcxPut(PcxFile,last,count)))return 0;
            total+=i;
            count=0;
            }
         }
      else
        {
            if(count)
            {
                if(!(i=PcxPut(PcxFile,last,count)))return 0;
                total+=i;
            }
            last=this;
            count=1;
        }
    }
    if(count)
    {
        if(!(i=PcxPut(PcxFile,last,count)))return 0;
        return (total+i);
    }

return total;
}

// to use SavePcx, the global sw screen information block is required
int SavePcx(char *lpszClipName)
{

    FILE *PcxFile;
    unsigned y,x=(sw.iBotx-sw.iTopx)+1;
    unsigned char *ptr;

    if((PcxFile=fopen(lpszClipName,"wb"))==NULL)return ESCKEY;

    pcxheader.xmin=sw.iTopx;
    pcxheader.ymin=sw.iTopy;
    pcxheader.xmax=sw.iBotx;
    pcxheader.ymax=sw.iBoty;
    pcxheader.bytes_per_line=x;
    pcxheader.hres =320;
    pcxheader.vres =200;
    fwrite((char *)&pcxheader.zsoft,sizeof(PCXHEADER),1,PcxFile);
    sw.iBoty++;
    ptr = (char *)&gib.lpScanLine[0];
    for(y=sw.iTopy;y<sw.iBoty;y++)
    {
        VesaRaster(ptr,sw.iTopx,y,x,READFLAG);
        WritePcxLine(PcxFile,ptr,x);
    }
    fwrite(sw.Pal,769,1,PcxFile);
    fclose(PcxFile);

return ENTERKEY;

}

// ======================================================================
// load either a partial or a full screen pcx
// ======================================================================
void ShowPcx(FILE *PcxFile,unsigned y1, unsigned y2,
                           unsigned x1,unsigned bytes)
{
    unsigned packet,offset,x,y;
    unsigned char count,byt;
    unsigned char far *ptr, *buf;

    ptr    = (char *)&gib.lpScanLine[0];
    buf    = (char *)&gib.lpScanLine[MAXSCANBUF/2];
    offset = MAXVGABUF;

    for(y=y1;y<y2;y++)
    {
    for(x=0;x<bytes;)
    {
       if(offset>=MAXVGABUF)
        {
           offset=0;
           fread(buf,MAXVGABUF,1,PcxFile);
        }
        count=1;
        byt=buf[offset];
        if(0xC0 == (0xC0 & byt))
        {
            offset++;
            if(offset>=MAXVGABUF)
            {
                offset=0;
                fread(buf,MAXVGABUF,1,PcxFile);
            }
            count= 0x3f & byt;
            byt  = buf[offset];
        }
        for(packet=0;packet<count;packet++)
        {
          ptr[x++]=byt;if(x>=(unsigned)bytes)break;
        }
        offset++;
        
    }
    VesaRaster(ptr,x1,y,bytes,WRITEFLAG);
    }
}

// ======================================================================
// load a pcx file into position using saved positional references
// returns 0  if successful
// returns 99 if not successful
// ======================================================================
int LoadPcx(char *lpszClipName)
{
    FILE *PcxFile;
    long dwSeekSize;
    unsigned char *ptr, temp;
    unsigned int x,y;

    // can't open
    if((PcxFile=fopen(lpszClipName,"rb"))==NULL)return -1;
    dwSeekSize = filelength(fileno(PcxFile))-769;

    // read header
    fread((char *)&pcxheader.zsoft,sizeof(PCXHEADER),1,PcxFile);

    // check for a full screen or less 256 color pcx file
    if(pcxheader.zsoft     != 10)return 99;


    if(pcxheader.version   != 5   ||
       pcxheader.pixbits   != 8   ||
       pcxheader.no_planes != 1   ||
       pcxheader.ymax      >  (gib.wDefaultHeight-1) ||
       pcxheader.xmax      >  (gib.wDefaultWidth-1)  ||
       pcxheader.bytes_per_line > gib.wDefaultWidth)
       {
         fclose(PcxFile);
         return -1;
        }

    ptr = (char *)&sw.Pal[0];
    fseek(PcxFile,dwSeekSize,SEEK_SET);
    fread(ptr,769,1,PcxFile);
    if(ptr[0]!=12)
    {
        fclose(PcxFile);return -2;
    }
    *ptr++;
    for(y=0;y<256;y++)
    {
      for(x=0;x<3;x++)
         {
          temp=*ptr++;
          gib.aRgbInfo[y][x]=(temp>>2);

         }
     }

    LoadPalette();

    fseek(PcxFile,128L,SEEK_SET);
    ShowPcx(PcxFile,pcxheader.ymin,pcxheader.ymax+1,
                    pcxheader.xmin,pcxheader.bytes_per_line);
    fclose(PcxFile);

return SUCCESS;
}
