// =========================================================================
// Program Name : VESASHOW.c
// Author       : Bill Buckels
// Date         : September 25, 1995
// Purpose      : Demonstration/Utility Program
//
//                Hardware Independent VESA Super VGA 256 Color or Less
//                Windows BMP File Loader using the VESALIB Graphics
//                Library Routines for VESA 256 Color Mode
//
//                This application displays Windows Bitmaps (BMP files)
//                in 256 color mode if VESA Bios is available.
//
//                Alternately, a Text Script listing BMP files and other
//                events can be used with VESASHOW as a slideshow.
//
//                .PCX files with 256 Colours are also supported.
//
//                Refer to the Source Code below for the Script format,
//                and more information about using VESALIB.
// =========================================================================

#include "vesalib.h"

// =========================================================================
// Delay Interrupt Handler stuff
// =========================================================================

void (interrupt _far *int1C)();
uint timerctr=0;
void interrupt timertick()
{
    _disable();
     timerctr++;
    _enable();
     int1C();
}

// =========================================================================
// The following function parses a line from the delimited script
// into its field components...
// This allows the script to be exported from any script writer program
// that can output delimited text.
// =========================================================================

#define FLIMIT 10
#define FLEN 80
char fld[FLIMIT][FLEN];

int GetFields(char *lpszBuf)
{
    unsigned wFields;
    char *ptr;
    int i, j;

    // Start with a clean record...
    wFields=1;
    for(i=0;i<10;i++)fld[i][0]=0;

    // Count the fields...
    for(i=0;lpszBuf[i]!=0;)
    {
       if(lpszBuf[i]==',')wFields++;
       if(lpszBuf[i]==10||lpszBuf[i]==13||lpszBuf[i]=='\x1a')lpszBuf[i]=0;
       else i++;
    }
    if(lpszBuf[0]==0)return 0;

    // Unpack the fields from the delimited string...
    j=0;
    if(wFields>10)wFields=FLIMIT;
    for(i=0;i<wFields;i++)
    {
      ptr=(char *)&lpszBuf[j];
      while(*ptr==' ')ptr++;
      while(*ptr=='\"')ptr++;
      while(lpszBuf[j]!=0)
      {
        if(lpszBuf[j]==',')
           {
              if(lpszBuf[j-1]=='\"')lpszBuf[j-1]=0;
              lpszBuf[j]=0;
           }
         else
           {
               j++;
           }
       }
       while(lpszBuf[j]==0)j++;
       strcpy((char *)&fld[i][0],ptr);
    }

return wFields;
}

// =========================================================================
// The following function reads a VESASHOW slideshow script
// =========================================================================

int BMPSlideShow(char *script)
{
   FILE *fp;
   int i;
   uint delay, action, color;
   char fnt[128], buf[512], *ptr;
   int x,y;

   bmp[0]=0;
   fnt[0]=0;

   if(script[0]!='@')
   {
       i=LoadVesaBitMap(script);
       if(i==EXECFAIL)
       {
         i=LoadPcx(script);
         if(i)
         {
            if(i==NOTPCX)return EXECFAIL;
            else return INITFAIL;
         }

       }
       if(i==SUCCESS)if(getch()==0)getch();
       return i;
   }

   if((fp=fopen((char *)&script[1],"r")) == NULL)return NOSCRIPT;

   if(fgets(buf,127,fp)==NULL)
   {
      fclose(fp);return EMPTY;
    }
   for(i=0;i<127;i++)buf[i]=toupper(buf[i]);
   if(strstr(buf,"VESASHOW")==NULL)
   {
      fclose(fp);return INVALID;
   }


   _disable();
   int1C= _dos_getvect(0x1C);
   _dos_setvect(0x1C,timertick);
   _enable();

   initmouse();

   for(;;)
   {
      wClsFlag=TRUE;
      if(kbhit())
      {
       if((c=getch())==0)getch();
       while(kbhit())if(getch()==0)getch();
       if(c==27)break;
      }

      action   =FALSE;
      gib.wXorigin = 0;
      gib.wYorigin = 0;

      if(fgets(buf,512,fp)==NULL)
      {
        rewind(fp);continue;
      }
      if(!GetFields(buf))continue;
      ptr=(unsigned char *)&fld[0][0];
      if(strcmpi(ptr,"DELAY")==0)action = DELAY;
      if(strcmpi(ptr,"TEXT")==0)action  = TEXT;
      if(strcmpi(ptr,"BMP")==0)action   = BMP;
      if(strcmpi(ptr,"PCX")==0)action   = PCX;

      switch(action)
      {
        case TEXT:    if(strcmpi((char *)&fld[1][0],fnt)!=0)
                      {
                        strcpy(fnt,(char *)&fld[1][0]);
                        if(LoadVesaFont(fnt)!=0)
                        {
                            fnt[0]=0;break;
                        }
                      }
                      gib.wXorigin = atoi((char *)&fld[2][0]);
                      gib.wYorigin = atoi((char *)&fld[3][0]);
                      color = atoi((char *)&fld[4][0]);
                      if(toupper(fld[4][0])=='B')color = gib.wBlack;
                      if(toupper(fld[4][0])=='W')color = gib.wWhite;
                      DrawVesaFont((unsigned char *)&fld[6][0],
                                    (int)gib.wXorigin,
                                    (int)gib.wYorigin,
                                    (char)toupper(fld[5][0]),
                                    color);
                      break;

        case DELAY:   delay = atoi((char *)&fld[1][0]);
                      delay *= 91;
                      delay /= 5;
                      if(delay==FALSE)
                      {
                       while(!kbhit())
                       {
                        if(mousebutton()==TRUE)break;
                       }
                       if((c=getch())==0)getch();
                       while(kbhit())if(getch()==0)getch();
                       break;
                      }
                      timerctr=0;
                      while(timerctr<delay)
                      {
                       if(mousebutton()==TRUE)break;
                       if(kbhit())
                        {
                           if((c=getch())==0)getch();
                           while(kbhit())if(getch()==0)getch();
                           break;
                         }
                      }
                      break;
         case BMP:
                      x=atoi((char *)&fld[2][0]);
                      y=atoi((char *)&fld[3][0]);
                      if(x!=-1)gib.wXorigin = x;
                      else gib.wXorigin = 0xffff;
                      if(y!=-1)gib.wYorigin = y;
                      else gib.wYorigin = 0xffff;
                      if(strcmpi("NONE",(char *)&fld[4][0])==0)
                                                     wClsFlag=FALSE;
                      LoadVesaBitMap((char *)&fld[1][0]);
                      break;

         case PCX:    LoadPcx((char *)&fld[1][0]);
                      break;



      }
      if(c==27)break;

   }
  fclose(fp);
  _disable();
  _dos_setvect(0x1C,int1C);
  _enable();

return SUCCESS;

}

// =========================================================================
// The following function sets up the vesa session
// =========================================================================

int VesaSession(char *lpszScript,int iWidth, int iHeight)
{
    int iRetVal;

    gib.wDefaultWidth  = SVGAWIDTH;
    gib.wDefaultHeight = SVGAHEIGHT;
    gib.wDefaultPlanes = SVGAPLANES;
    gib.wDefaultBits   = SVGABITS;

    if(iWidth>0 )gib.wDefaultWidth  = iWidth;
    if(iHeight>0)gib.wDefaultHeight = iHeight;

    if((gib.wVesaMode =
        FindVesaMode(gib.wDefaultWidth,gib.wDefaultHeight,
                     gib.wDefaultPlanes,gib.wDefaultBits))==FALSE)
                                                      return UNAVAILABLE;

    gib.wSaveMode = GetVesaMode();
    if(SetVesaMode(gib.wVesaMode)!=gib.wVesaMode)
      {
         SetVesaMode(gib.wSaveMode); return UNSUPPORTED;
      }

    if((iRetVal=InitVesaSession())==SUCCESS)
    {
       iRetVal = BMPSlideShow(lpszScript);
    }
    SetVesaMode(gib.wSaveMode);


return iRetVal;
}

// =========================================================================
// The program mainline checks for a vesa driver and
// if found calls the vesa setup...
// =========================================================================

char *lpTitle=
"VESASHOW(C) Bill Buckels Copyright 1995\n"
"Hardware Independent BMP file loader and script interpreter";

int main(int argc, char **argv)
{
    FILE *fp;
    char szBuf[81];
    int i,bQuiet,iRetVal,iWidth,iHeight;

    // Check to see if QUIET mode is on...
    // If so return only the error as an exit code...
    // and don't display any messages.
    bQuiet = FALSE;
    if(argc>2)
    {
       for(i=1;i<argc;i++)if(strcmpi(argv[i],"quiet")==0)bQuiet = TRUE;
    }

    // If a VESA driver is not loaded,
    // There is no point in continuing.
    if((iRetVal = GetVgaInfo())!=SUCCESS)
    {
      if(!bQuiet)puts(lpTitle);

      switch(iRetVal)
      {
      case INITFAIL:
        if(!bQuiet)
            printf("VESA Super VGA Information not supported.\n");
            return 1;
      case EXECFAIL:
        if(!bQuiet)
            printf("VESA Super VGA Information failed.\n");
            return 2;
      }

    }

    // if we are not in quiet mode and no file name was entered
    // prompt for a file name.
    iWidth  = 0;
    iHeight = 0;
    if(argc<2)
    {
    if(bQuiet)return 3;
    puts(lpTitle);
    printf("Usage 1:\"VESASHOW any.bmp (widthoption) (heightoption)\"\n");
    printf("      2:\"VESASHOW @script.txt (widthoption) (heightoption)\"\n");
    printf("      3:\"VESASHOW ... QUIET\"\n");
    printf("\nEnter Script or Bmp Name : ");gets(szBuf);
    if(szBuf[0]==0)return 4;
    }
    else
    {
    strcpy(szBuf,argv[1]);
    if(argc>3)
      {
      iWidth=atoi(argv[2]);
      iHeight=atoi(argv[3]);
      }
    }


    // Execute the vesa session...
    // if there is a problem, report the problem as an exit code
    // after displaying the message (if not in quiet mode)
    if((iRetVal=VesaSession(szBuf,iWidth,iHeight))!=SUCCESS)
    {
      if(!bQuiet)puts(lpTitle);
      switch(iRetVal)
      {
      case UNAVAILABLE:
         if(!bQuiet)
         printf("VESA mode %d x %d x 256 colors is not available\n",
                    gib.wDefaultWidth,gib.wDefaultHeight);
            return 5;
      case UNSUPPORTED:
         if(!bQuiet)
         printf("VESA Configuration %d x %d x 256 colors is not supported\n",
                    gib.wDefaultWidth,gib.wDefaultHeight);
            return 6;
      case NOMEM:
         if(!bQuiet)
            printf("No Memory...\n");
            return 7;
      case INITFAIL:
         if(!bQuiet)
            perror(szBuf);
            return 8;
      case EXECFAIL:
         if(!bQuiet)
            printf("%s is not a supported format.\n",szBuf);
            return 9;
      case NOSCRIPT:
         if(!bQuiet)
            printf("Problem opening VESASHOW script %s.\n",szBuf);
            return 10;
      case EMPTY:
         if(!bQuiet)
            printf("VESASHOW script %s is empty!\n",szBuf);
            return 11;
      case INVALID:
         if(!bQuiet)
            printf("VESASHOW script %s is in the wrong format.\n",szBuf);
            return 12;
      }
    }

return SUCCESS;
}
